@@ -1,11 +1,12 @@
 from __future__ import annotations
 
+import operator
 import warnings
-from typing import TYPE_CHECKING, Any, cast
+from typing import TYPE_CHECKING, Any, Callable, Literal, cast
 
 import numpy as np
 
-from narwhals._compliant import EagerSeries
+from narwhals._compliant import EagerSeries, EagerSeriesHist
 from narwhals._pandas_like.series_cat import PandasLikeSeriesCatNamespace
 from narwhals._pandas_like.series_dt import PandasLikeSeriesDateTimeNamespace
 from narwhals._pandas_like.series_list import PandasLikeSeriesListNamespace
@@ -34,9 +35,10 @@ if TYPE_CHECKING:
     import pandas as pd
     import polars as pl
     import pyarrow as pa
-    from typing_extensions import Self, TypeIs
+    from typing_extensions import Self, TypeAlias, TypeIs
 
     from narwhals._arrow.typing import ChunkedArrayAny
+    from narwhals._compliant.series import HistData
     from narwhals._pandas_like.dataframe import PandasLikeDataFrame
     from narwhals._pandas_like.namespace import PandasLikeNamespace
     from narwhals._utils import Version, _LimitedContext
@@ -46,17 +48,19 @@ if TYPE_CHECKING:
         FillNullStrategy,
         Into1DArray,
         IntoDType,
+        ModeKeepStrategy,
         NonNestedLiteral,
-        NumericLiteral,
+        PythonLiteral,
         RankMethod,
         RollingInterpolationMethod,
         SizedMultiIndexSelector,
-        TemporalLiteral,
         _1DArray,
-        _AnyDArray,
         _SliceIndex,
     )
 
+    PandasHistData: TypeAlias = "HistData[pd.Series[Any], list[float]]"
+
+
 PANDAS_TO_NUMPY_DTYPE_NO_MISSING = {
     "Int64": "int64",
     "int64[pyarrow]": "int64",
@@ -196,7 +200,7 @@ class PandasLikeSeries(EagerSeries[Any]):
 
     @classmethod
     def _align_full_broadcast(cls, *series: Self) -> Sequence[Self]:
-        Series = series[0].__native_namespace__().Series  # noqa: N806
+        Series = series[0].__native_namespace__().Series
         lengths = [len(s) for s in series]
         max_length = max(lengths)
         idx = series[lengths.index(max_length)].native.index
@@ -236,8 +240,7 @@ class PandasLikeSeries(EagerSeries[Any]):
             import numpy as np
 
             return np
-        else:
-            return import_array_module(self._implementation)
+        return import_array_module(self._implementation)
 
     def ewm_mean(
         self,
@@ -283,9 +286,9 @@ class PandasLikeSeries(EagerSeries[Any]):
         return self._with_native(s)
 
     def _scatter_in_place(self, indices: Self, values: Self) -> None:
+        # Scatter, modifying original Series. Use with care!
         implementation = self._implementation
         backend_version = self._backend_version
-        # Scatter, modifying original Series. Use with care!
         values_native = set_index(
             values.native,
             self.native.index[indices.native],
@@ -300,6 +303,11 @@ class PandasLikeSeries(EagerSeries[Any]):
             self.native.iloc[indices.native] = values_native
 
     def cast(self, dtype: IntoDType) -> Self:
+        if self.dtype == dtype and self.native.dtype != "object":
+            # Avoid dealing with pandas' type-system if we can. Note that it's only
+            # safe to do this if we're not starting with object dtype, see tests/expr_and_series/cast_test.py::test_cast_object_pandas
+            # for an example of why.
+            return self._with_native(self.native, preserve_broadcast=True)
         pd_dtype = narwhals_to_native_dtype(
             dtype,
             dtype_backend=get_dtype_backend(self.native.dtype, self._implementation),
@@ -308,7 +316,7 @@ class PandasLikeSeries(EagerSeries[Any]):
         )
         return self._with_native(self.native.astype(pd_dtype), preserve_broadcast=True)
 
-    def item(self, index: int | None) -> Any:
+    def item(self, index: int | None = None) -> Any:
         # cuDF doesn't have Series.item().
         if index is None:
             if len(self) != 1:
@@ -377,103 +385,93 @@ class PandasLikeSeries(EagerSeries[Any]):
             other_native = predicate
         return self._with_native(self.native.loc[other_native]).alias(self.name)
 
+    def first(self) -> PythonLiteral:
+        return self.native.iloc[0] if len(self.native) else None
+
+    def last(self) -> PythonLiteral:
+        return self.native.iloc[-1] if len(self.native) else None
+
+    def _with_binary(self, op: Callable[..., PandasLikeSeries], other: Any) -> Self:
+        ser, other_native = align_and_extract_native(self, other)
+        preserve_broadcast = self._broadcast and getattr(other, "_broadcast", True)
+        return self._with_native(
+            op(ser, other_native), preserve_broadcast=preserve_broadcast
+        ).alias(self.name)
+
+    def _with_binary_right(self, op: Callable[..., PandasLikeSeries], other: Any) -> Self:
+        return self._with_binary(lambda x, y: op(y, x), other).alias(self.name)
+
     def __eq__(self, other: object) -> Self:  # type: ignore[override]
-        ser, other = align_and_extract_native(self, other)
-        return self._with_native(ser == other).alias(self.name)
+        return self._with_binary(operator.eq, other)
 
     def __ne__(self, other: object) -> Self:  # type: ignore[override]
-        ser, other = align_and_extract_native(self, other)
-        return self._with_native(ser != other).alias(self.name)
+        return self._with_binary(operator.ne, other)
 
     def __ge__(self, other: Any) -> Self:
-        ser, other = align_and_extract_native(self, other)
-        return self._with_native(ser >= other).alias(self.name)
+        return self._with_binary(operator.ge, other)
 
     def __gt__(self, other: Any) -> Self:
-        ser, other = align_and_extract_native(self, other)
-        return self._with_native(ser > other).alias(self.name)
+        return self._with_binary(operator.gt, other)
 
     def __le__(self, other: Any) -> Self:
-        ser, other = align_and_extract_native(self, other)
-        return self._with_native(ser <= other).alias(self.name)
+        return self._with_binary(operator.le, other)
 
     def __lt__(self, other: Any) -> Self:
-        ser, other = align_and_extract_native(self, other)
-        return self._with_native(ser < other).alias(self.name)
+        return self._with_binary(operator.lt, other)
 
     def __and__(self, other: Any) -> Self:
-        ser, other = align_and_extract_native(self, other)
-        return self._with_native(ser & other).alias(self.name)
+        return self._with_binary(operator.and_, other)
 
     def __rand__(self, other: Any) -> Self:
-        ser, other = align_and_extract_native(self, other)
-        ser = cast("pd.Series[Any]", ser)
-        return self._with_native(ser.__and__(other)).alias(self.name)
+        return self._with_binary_right(operator.and_, other)
 
     def __or__(self, other: Any) -> Self:
-        ser, other = align_and_extract_native(self, other)
-        return self._with_native(ser | other).alias(self.name)
+        return self._with_binary(operator.or_, other)
 
     def __ror__(self, other: Any) -> Self:
-        ser, other = align_and_extract_native(self, other)
-        ser = cast("pd.Series[Any]", ser)
-        return self._with_native(ser.__or__(other)).alias(self.name)
+        return self._with_binary_right(operator.or_, other)
 
     def __add__(self, other: Any) -> Self:
-        ser, other = align_and_extract_native(self, other)
-        return self._with_native(ser + other).alias(self.name)
+        return self._with_binary(operator.add, other)
 
     def __radd__(self, other: Any) -> Self:
-        _, other_native = align_and_extract_native(self, other)
-        return self._with_native(self.native.__radd__(other_native)).alias(self.name)
+        return self._with_binary_right(operator.add, other)
 
     def __sub__(self, other: Any) -> Self:
-        ser, other = align_and_extract_native(self, other)
-        return self._with_native(ser - other).alias(self.name)
+        return self._with_binary(operator.sub, other)
 
     def __rsub__(self, other: Any) -> Self:
-        _, other_native = align_and_extract_native(self, other)
-        return self._with_native(self.native.__rsub__(other_native)).alias(self.name)
+        return self._with_binary_right(operator.sub, other)
 
     def __mul__(self, other: Any) -> Self:
-        ser, other = align_and_extract_native(self, other)
-        return self._with_native(ser * other).alias(self.name)
+        return self._with_binary(operator.mul, other)
 
     def __rmul__(self, other: Any) -> Self:
-        _, other_native = align_and_extract_native(self, other)
-        return self._with_native(self.native.__rmul__(other_native)).alias(self.name)
+        return self._with_binary_right(operator.mul, other)
 
     def __truediv__(self, other: Any) -> Self:
-        ser, other = align_and_extract_native(self, other)
-        return self._with_native(ser / other).alias(self.name)
+        return self._with_binary(operator.truediv, other)
 
     def __rtruediv__(self, other: Any) -> Self:
-        _, other_native = align_and_extract_native(self, other)
-        return self._with_native(self.native.__rtruediv__(other_native)).alias(self.name)
+        return self._with_binary_right(operator.truediv, other)
 
     def __floordiv__(self, other: Any) -> Self:
-        ser, other = align_and_extract_native(self, other)
-        return self._with_native(ser // other).alias(self.name)
+        return self._with_binary(operator.floordiv, other)
 
     def __rfloordiv__(self, other: Any) -> Self:
-        _, other_native = align_and_extract_native(self, other)
-        return self._with_native(self.native.__rfloordiv__(other_native)).alias(self.name)
+        return self._with_binary_right(operator.floordiv, other)
 
     def __pow__(self, other: Any) -> Self:
-        ser, other = align_and_extract_native(self, other)
-        return self._with_native(ser**other).alias(self.name)
+        return self._with_binary(operator.pow, other)
 
     def __rpow__(self, other: Any) -> Self:
-        _, other_native = align_and_extract_native(self, other)
-        return self._with_native(self.native.__rpow__(other_native)).alias(self.name)
+        return self._with_binary_right(operator.pow, other)
 
     def __mod__(self, other: Any) -> Self:
-        ser, other = align_and_extract_native(self, other)
-        return self._with_native(ser % other).alias(self.name)
+        return self._with_binary(operator.mod, other)
 
     def __rmod__(self, other: Any) -> Self:
-        _, other_native = align_and_extract_native(self, other)
-        return self._with_native(self.native.__rmod__(other_native)).alias(self.name)
+        return self._with_binary_right(operator.mod, other)
 
     # Unary
 
@@ -519,27 +517,25 @@ class PandasLikeSeries(EagerSeries[Any]):
         ser_not_null = self.native.dropna()
         if len(ser_not_null) == 0:
             return None
-        elif len(ser_not_null) == 1:
+        if len(ser_not_null) == 1:
             return float("nan")
-        elif len(ser_not_null) == 2:
+        if len(ser_not_null) == 2:
             return 0.0
-        else:
-            m = ser_not_null - ser_not_null.mean()
-            m2 = (m**2).mean()
-            m3 = (m**3).mean()
-            return m3 / (m2**1.5) if m2 != 0 else float("nan")
+        m = ser_not_null - ser_not_null.mean()
+        m2 = (m**2).mean()
+        m3 = (m**3).mean()
+        return m3 / (m2**1.5) if m2 != 0 else float("nan")
 
     def kurtosis(self) -> float | None:
         ser_not_null = self.native.dropna()
         if len(ser_not_null) == 0:
             return None
-        elif len(ser_not_null) == 1:
+        if len(ser_not_null) == 1:
             return float("nan")
-        else:
-            m = ser_not_null - ser_not_null.mean()
-            m2 = (m**2).mean()
-            m4 = (m**4).mean()
-            return m4 / (m2**2) - 3.0 if m2 != 0 else float("nan")
+        m = ser_not_null - ser_not_null.mean()
+        m2 = (m**2).mean()
+        m4 = (m**4).mean()
+        return m4 / (m2**2) - 3.0 if m2 != 0 else float("nan")
 
     def len(self) -> int:
         return len(self.native)
@@ -551,10 +547,11 @@ class PandasLikeSeries(EagerSeries[Any]):
 
     def is_nan(self) -> Self:
         ser = self.native
-        if self.dtype.is_numeric():
-            return self._with_native(ser != ser, preserve_broadcast=True)  # noqa: PLR0124
-        msg = f"`.is_nan` only supported for numeric dtype and not {self.dtype}, did you mean `.is_null`?"
-        raise InvalidOperationError(msg)
+        if not self.dtype.is_numeric():
+            msg = f"`.is_nan` only supported for numeric dtype and not {self.dtype}, did you mean `.is_null`?"
+            raise InvalidOperationError(msg)
+        # If/when pandas exposes an API which distinguishes NaN vs null, use that.
+        return self._with_native(ser != ser, preserve_broadcast=True)  # noqa: PLR0124
 
     def fill_null(
         self,
@@ -587,6 +584,18 @@ class PandasLikeSeries(EagerSeries[Any]):
                 )
         return res_ser
 
+    def fill_nan(self, value: float | None) -> Self:
+        if not self.dtype.is_numeric():  # pragma: no cover
+            msg = f"`.fill_nan` only supported for numeric dtype and not {self.dtype}, did you mean `.fill_null`?"
+            raise InvalidOperationError(msg)
+        s = self.native
+        fill = s.array.dtype.na_value if value is None else value
+        # If/when pandas exposes an API which distinguishes NaN vs null, use that.
+        mask = s != s  # noqa: PLR0124
+        # Carefully use `inplace`, as `mask` isn't provided by the user.
+        mask.fillna(False, inplace=True)  # noqa: PD002
+        return self._with_native(s.mask(mask, fill), preserve_broadcast=True)
+
     def drop_nulls(self) -> Self:
         return self._with_native(self.native.dropna())
 
@@ -703,9 +712,9 @@ class PandasLikeSeries(EagerSeries[Any]):
     def to_pandas(self) -> pd.Series[Any]:
         if self._implementation is Implementation.PANDAS:
             return self.native
-        elif self._implementation is Implementation.CUDF:  # pragma: no cover
+        if self._implementation is Implementation.CUDF:  # pragma: no cover
             return self.native.to_pandas()
-        elif self._implementation is Implementation.MODIN:
+        if self._implementation is Implementation.MODIN:
             return self.native._to_pandas()
         msg = f"Unknown implementation: {self._implementation}"  # pragma: no cover
         raise AssertionError(msg)
@@ -735,8 +744,7 @@ class PandasLikeSeries(EagerSeries[Any]):
 
         if descending:
             return self.native.is_monotonic_decreasing
-        else:
-            return self.native.is_monotonic_increasing
+        return self.native.is_monotonic_increasing
 
     def value_counts(
         self, *, sort: bool, parallel: bool, name: str | None, normalize: bool
@@ -778,6 +786,62 @@ class PandasLikeSeries(EagerSeries[Any]):
     def round(self, decimals: int) -> Self:
         return self._with_native(self.native.round(decimals=decimals))
 
+    def floor(self) -> Self:
+        native = self.native
+        native_cls = type(native)
+        implementation = self._implementation
+        if get_dtype_backend(native.dtype, implementation=implementation) == "pyarrow":
+            import pyarrow.compute as pc
+
+            from narwhals._arrow.utils import native_to_narwhals_dtype
+
+            ca = native.array._pa_array
+            result_arr = cast("ChunkedArrayAny", pc.floor(ca))
+            nw_dtype = native_to_narwhals_dtype(result_arr.type, self._version)
+            out_dtype = narwhals_to_native_dtype(
+                nw_dtype, "pyarrow", self._implementation, self._version
+            )
+            result_native = native_cls(
+                result_arr, dtype=out_dtype, index=native.index, name=native.name
+            )
+        else:
+            array_funcs = self._array_funcs
+            result_arr = array_funcs.floor(self.native)
+            result_native = (
+                native_cls(result_arr, index=native.index, name=native.name)
+                if implementation.is_cudf()
+                else result_arr
+            )
+        return self._with_native(result_native)
+
+    def ceil(self) -> Self:
+        native = self.native
+        native_cls = type(native)
+        implementation = self._implementation
+        if get_dtype_backend(native.dtype, implementation=implementation) == "pyarrow":
+            import pyarrow.compute as pc
+
+            from narwhals._arrow.utils import native_to_narwhals_dtype
+
+            ca = native.array._pa_array
+            result_arr = cast("ChunkedArrayAny", pc.ceil(ca))
+            nw_dtype = native_to_narwhals_dtype(result_arr.type, self._version)
+            out_dtype = narwhals_to_native_dtype(
+                nw_dtype, "pyarrow", self._implementation, self._version
+            )
+            result_native = native_cls(
+                result_arr, dtype=out_dtype, index=native.index, name=native.name
+            )
+        else:
+            array_funcs = self._array_funcs
+            result_arr = array_funcs.ceil(self.native)
+            result_native = (
+                native_cls(result_arr, index=native.index, name=native.name)
+                if implementation.is_cudf()
+                else result_arr
+            )
+        return self._with_native(result_native)
+
     def to_dummies(self, *, separator: str, drop_first: bool) -> PandasLikeDataFrame:
         from narwhals._pandas_like.dataframe import PandasLikeDataFrame
 
@@ -810,19 +874,55 @@ class PandasLikeSeries(EagerSeries[Any]):
     def gather_every(self, n: int, offset: int) -> Self:
         return self._with_native(self.native.iloc[offset::n])
 
-    def clip(
-        self,
-        lower_bound: Self | NumericLiteral | TemporalLiteral | None,
-        upper_bound: Self | NumericLiteral | TemporalLiteral | None,
-    ) -> Self:
-        _, lower = (
-            align_and_extract_native(self, lower_bound) if lower_bound else (None, None)
-        )
-        _, upper = (
-            align_and_extract_native(self, upper_bound) if upper_bound else (None, None)
-        )
-        kwargs = {"axis": 0} if self._implementation is Implementation.MODIN else {}
-        return self._with_native(self.native.clip(lower, upper, **kwargs))
+    def clip(self, lower_bound: Self, upper_bound: Self) -> Self:
+        _, lower = align_and_extract_native(self, lower_bound)
+        _, upper = align_and_extract_native(self, upper_bound)
+        impl = self._implementation
+        kwargs: dict[str, Any] = {"axis": 0} if impl.is_modin() else {}
+        result = self.native
+
+        if not impl.is_pandas():
+            # Workaround for both cudf and modin when clipping with a series
+            #   * cudf: https://github.com/rapidsai/cudf/issues/17682
+            #   * modin: https://github.com/modin-project/modin/issues/7415
+            if self._is_native(lower):
+                result = result.where(result >= lower, lower)
+                lower = None
+            if self._is_native(upper):
+                result = result.where(result <= upper, upper)
+                upper = None
+
+        return self._with_native(result.clip(lower, upper, **kwargs))
+
+    def clip_lower(self, lower_bound: Self) -> Self:
+        _, lower = align_and_extract_native(self, lower_bound)
+        impl = self._implementation
+        kwargs: dict[str, Any] = {"axis": 0} if impl.is_modin() else {}
+        result = self.native
+
+        if not impl.is_pandas() and self._is_native(lower):  # pragma: no cover
+            # Workaround for both cudf and modin when clipping with a series
+            #   * cudf: https://github.com/rapidsai/cudf/issues/17682
+            #   * modin: https://github.com/modin-project/modin/issues/7415
+            result = result.where(result >= lower, lower)
+            lower = None
+
+        return self._with_native(result.clip(lower, **kwargs))
+
+    def clip_upper(self, upper_bound: Self) -> Self:
+        _, upper = align_and_extract_native(self, upper_bound)
+        impl = self._implementation
+        kwargs: dict[str, Any] = {"axis": 0} if impl.is_modin() else {}
+        result = self.native
+
+        if not impl.is_pandas() and self._is_native(upper):  # pragma: no cover
+            # Workaround for both cudf and modin when clipping with a series
+            #   * cudf: https://github.com/rapidsai/cudf/issues/17682
+            #   * modin: https://github.com/modin-project/modin/issues/7415
+            result = result.where(result <= upper, upper)
+            upper = None
+
+        return self._with_native(result.clip(upper=upper, **kwargs))
 
     def to_arrow(self) -> pa.Array[Any]:
         if self._implementation is Implementation.CUDF:
@@ -832,10 +932,10 @@ class PandasLikeSeries(EagerSeries[Any]):
 
         return pa.Array.from_pandas(self.native)
 
-    def mode(self) -> Self:
+    def mode(self, *, keep: ModeKeepStrategy) -> Self:
         result = self.native.mode()
         result.name = self.name
-        return self._with_native(result)
+        return self._with_native(result.head(1) if keep == "any" else result)
 
     def cum_count(self, *, reverse: bool) -> Self:
         not_na_series = ~self.native.isna()
@@ -945,77 +1045,23 @@ class PandasLikeSeries(EagerSeries[Any]):
             )
         )
 
-    def hist(  # noqa: C901, PLR0912
-        self,
-        bins: list[float | int] | None,
-        *,
-        bin_count: int | None,
-        include_breakpoint: bool,
+    def hist_from_bins(
+        self, bins: list[float], *, include_breakpoint: bool
     ) -> PandasLikeDataFrame:
-        from numpy import linspace, zeros
-
-        from narwhals._pandas_like.dataframe import PandasLikeDataFrame
-
-        ns = self.__native_namespace__()
-        data: dict[str, Sequence[int | float | str] | _AnyDArray]
-
-        if bin_count == 0 or (bins is not None and len(bins) <= 1):
-            data = {}
-            if include_breakpoint:
-                data["breakpoint"] = []
-            data["count"] = []
-            return PandasLikeDataFrame.from_native(ns.DataFrame(data), context=self)
-
-        if self.native.count() < 1:
-            if bins is not None:
-                data = {"breakpoint": bins[1:], "count": zeros(shape=len(bins) - 1)}
-            else:
-                count = cast("int", bin_count)
-                if bin_count == 1:
-                    data = {"breakpoint": [1.0], "count": [0]}
-                else:
-                    data = {
-                        "breakpoint": linspace(0, 1, count + 1)[1:],
-                        "count": zeros(shape=count),
-                    }
-            if not include_breakpoint:
-                del data["breakpoint"]
-            return PandasLikeDataFrame.from_native(ns.DataFrame(data), context=self)
-
-        if bin_count is not None:
-            # use Polars binning behavior
-            lower, upper = self.native.min(), self.native.max()
-            if lower == upper:
-                lower -= 0.5
-                upper += 0.5
-
-            if bin_count == 1:
-                data = {"breakpoint": [upper], "count": [self.native.count()]}
-                if not include_breakpoint:
-                    del data["breakpoint"]
-                return PandasLikeDataFrame.from_native(ns.DataFrame(data), context=self)
-
-            bins = linspace(lower, upper, bin_count + 1)
-            bin_count = None
-
-        # pandas (2.2.*) .value_counts(bins=int) adjusts the lowest bin twice, result in improper counts.
-        # pandas (2.2.*) .value_counts(bins=[...]) adjusts the lowest bin which should not happen since
-        #   the bins were explicitly passed in.
-        categories = ns.cut(
-            self.native,
-            bins=bins if bin_count is None else bin_count,
-            include_lowest=True,  # Polars 1.27.0 always includes the lowest bin
+        return (
+            _PandasHist.from_series(self, include_breakpoint=include_breakpoint)
+            .with_bins(bins)
+            .to_frame()
         )
-        # modin (0.32.0) .value_counts(...) silently drops bins with empty observations, .reindex
-        #   is necessary to restore these bins.
-        result = categories.value_counts(dropna=True, sort=False).reindex(
-            categories.cat.categories, fill_value=0
+
+    def hist_from_bin_count(
+        self, bin_count: int, *, include_breakpoint: bool
+    ) -> PandasLikeDataFrame:
+        return (
+            _PandasHist.from_series(self, include_breakpoint=include_breakpoint)
+            .with_bin_count(bin_count)
+            .to_frame()
         )
-        data = {}
-        if include_breakpoint:
-            data["breakpoint"] = bins[1:] if bins is not None else result.index.right
-        data["count"] = result.reset_index(drop=True)
-        return PandasLikeDataFrame.from_native(ns.DataFrame(data), context=self)
 
     def log(self, base: float) -> Self:
         native = self.native
@@ -1104,3 +1150,80 @@ class PandasLikeSeries(EagerSeries[Any]):
             msg = "Series must be of PyArrow Struct type to support struct namespace."
             raise TypeError(msg)
         return PandasLikeSeriesStructNamespace(self)
+
+
+class _PandasHist(EagerSeriesHist["pd.Series[Any]", "list[float]"]):
+    _series: PandasLikeSeries
+
+    def to_frame(self) -> PandasLikeDataFrame:
+        from_native = self._series.__narwhals_namespace__()._dataframe.from_native
+        DataFrame = self._series.__native_namespace__().DataFrame
+        return from_native(DataFrame(self._data), context=self._series)
+
+    # NOTE: *Could* be handled at narwhals-level
+    def is_empty_series(self) -> bool:
+        return self._series.count() < 1
+
+    # NOTE: *Could* be handled at narwhals-level, **iff** we add `nw.repeat`, `nw.linear_space`
+    # See https://github.com/narwhals-dev/narwhals/pull/2839#discussion_r2215630696
+    def series_empty(self, arg: int | list[float], /) -> PandasHistData:
+        count = self._zeros(arg)
+        if self._breakpoint:
+            return {"breakpoint": self._calculate_breakpoint(arg), "count": count}
+        return {"count": count}
+
+    def _zeros(self, arg: int | list[float], /) -> _1DArray:
+        zeros = self._series._array_funcs.zeros
+        return zeros(arg) if isinstance(arg, int) else zeros(len(arg) - 1)
+
+    # NOTE: Based on `pl.Expr.cut`
+    def _cut(
+        self,
+        breaks: list[float] | _1DArray,
+        *,
+        labels: Sequence[str] | None = None,
+        closed: Literal["left", "right"] = "right",
+    ) -> pd.Series[Any]:
+        # NOTE: Polars 1.27.0 always includes the lowest bin
+        cut = self._series.__native_namespace__().cut
+        return cut(
+            self.native,
+            bins=breaks,
+            right=closed == "right",
+            labels=labels,
+            include_lowest=True,
+        )
+
+    def _linear_space(
+        self,
+        start: float,
+        end: float,
+        num_samples: int,
+        *,
+        closed: Literal["both", "none"] = "both",
+    ) -> _1DArray:
+        return self._series._array_funcs.linspace(
+            start=start, stop=end, num=num_samples, endpoint=closed == "both"
+        )
+
+    def _calculate_bins(self, bin_count: int) -> _1DArray:
+        """Prepare bins for histogram calculation from bin_count."""
+        lower, upper = self.native.min(), self.native.max()
+        if lower == upper:
+            lower -= 0.5
+            upper += 0.5
+        return self._linear_space(lower, upper, bin_count + 1)
+
+    def _calculate_hist(self, bins: list[float] | _1DArray) -> PandasHistData:
+        # pandas (2.2.*) .value_counts(bins=[...]) adjusts the lowest bin which should not
+        #   happen since the bins were explicitly passed in.
+        categories = self._cut(bins)
+        # modin (0.32.0) .value_counts(...) silently drops bins with empty observations,
+        #   .reindex is necessary to restore these bins.
+        count = categories.value_counts(dropna=True, sort=False).reindex(
+            categories.cat.categories, fill_value=0
+        )
+        count.reset_index(drop=True, inplace=True)  # noqa: PD002
+        if self._breakpoint:
+            return {"breakpoint": bins[1:], "count": count}
+        return {"count": count}
