@@ -130,6 +130,7 @@ def cudf_constructor(obj: Data) -> IntoDataFrame:  # pragma: no cover
 
 
 def polars_eager_constructor(obj: Data) -> pl.DataFrame:
+    pytest.importorskip("polars")
     import polars as pl
 
     return pl.DataFrame(obj)
@@ -222,7 +223,7 @@ def ibis_lazy_constructor(obj: Data) -> ibis.Table:  # pragma: no cover
     pytest.importorskip("polars")
     import polars as pl
 
-    ldf = pl.from_dict(obj).lazy()
+    ldf = pl.LazyFrame(obj)
     table_name = str(uuid.uuid4())
     return _ibis_backend().create_table(table_name, ldf)
 
