@@ -5,7 +5,6 @@ from datetime import date
 from typing import TYPE_CHECKING, Any
 
 import numpy as np
-import pyarrow as pa
 import pytest
 
 import narwhals as nw
@@ -122,6 +121,8 @@ def test_date_lit(constructor: Constructor, request: pytest.FixtureRequest) -> N
         "cudf" in str(constructor) and CUDF_VERSION >= (25, 8, 0)
     ):
         request.applymarker(pytest.mark.xfail)
+    if "pandas" in str(constructor):
+        pytest.importorskip("pyarrow")
     df = nw.from_native(constructor({"a": [1]}))
     result = df.with_columns(nw.lit(date(2020, 1, 1), dtype=nw.Date)).collect_schema()
     if df.implementation.is_cudf():
@@ -132,6 +133,9 @@ def test_date_lit(constructor: Constructor, request: pytest.FixtureRequest) -> N
 
 
 def test_pyarrow_lit_string() -> None:
+    pytest.importorskip("pyarrow")
+    import pyarrow as pa
+
     df = nw.from_native(pa.table({"a": [1, 2, 3]}))
     result = df.select(nw.lit("foo")).to_native().schema.field("literal")
     assert pa.types.is_string(result.type)
