@@ -371,7 +371,7 @@ def test_group_by_shift_raises(constructor: Constructor) -> None:
     df_native = {"a": [1, 2, 3], "b": [1, 1, 2]}
     df = nw.from_native(constructor(df_native))
     with pytest.raises(InvalidOperationError, match="does not aggregate"):
-        df.group_by("b").agg(nw.col("a").shift(1))
+        df.group_by("b").agg(nw.col("a").abs())
 
 
 def test_double_same_aggregation(
@@ -534,7 +534,7 @@ def test_group_by_raise_if_not_preserves_length(
 ) -> None:
     data = {"a": [1, 2, 2, None], "b": [0, 1, 2, 3], "x": [1, 2, 3, 4]}
     df = nw.from_native(constructor(data))
-    with pytest.raises(InvalidOperationError):
+    with pytest.raises((InvalidOperationError, NotImplementedError)):
         df.group_by(keys).agg(nw.col("x").max())
 
 
@@ -772,3 +772,31 @@ def test_group_by_agg_last(
         df = df.sort(aggs, **pre_sort)
     result = df.group_by(keys).agg(nw.col(aggs).last()).sort(keys)
     assert_equal_data(result, expected)
+
+
+def test_multi_column_expansion(constructor: Constructor) -> None:
+    if "polars" in str(constructor) and POLARS_VERSION < (1, 32):
+        pytest.skip(reason="https://github.com/pola-rs/polars/issues/21773")
+    if "modin" in str(constructor):
+        pytest.skip(reason="Internal error")
+    df = nw.from_native(constructor({"a": [1, 1, 2], "b": [4, 5, 6]}))
+    result = (
+        df.group_by("a")
+        .agg(nw.all().sum().name.suffix("_aggregated"))
+        .sort("a", descending=True)
+    )
+    expected = {"a": [2, 1], "b_aggregated": [6, 9]}
+    assert_equal_data(result, expected)
+    result = (
+        df.group_by("a")
+        .agg(nw.col("a", "b").sum().name.suffix("_aggregated"))
+        .sort("a", descending=True)
+    )
+    expected = {"a": [2, 1], "a_aggregated": [2, 2], "b_aggregated": [6, 9]}
+    assert_equal_data(result, expected)
+    result = (
+        df.group_by("a")
+        .agg(nw.nth(0, 1).sum().name.suffix("_aggregated"))
+        .sort("a", descending=True)
+    )
+    assert_equal_data(result, expected)
