@@ -17,11 +17,11 @@ from tests.utils import (
 )
 
 if TYPE_CHECKING:
-    from narwhals.typing import IntoLazyFrameT, JoinStrategy, NativeDataFrame
+    from narwhals.typing import IntoDataFrame, IntoLazyFrameT, JoinStrategy
 
 
 def from_native_lazy(
-    native: IntoLazyFrameT | NativeDataFrame,
+    native: IntoLazyFrameT | IntoDataFrame,
 ) -> nw.LazyFrame[IntoLazyFrameT] | nw.LazyFrame[Any]:
     """Every join test [needs to use `.lazy()` for typing]*.
 
@@ -777,6 +777,8 @@ def test_join_duplicate_column_names(
         # need to investigate.
     ):
         request.applymarker(pytest.mark.xfail)
+    data = {"a": [1, 2, 3, 4, 5], "b": [6, 6, 6, 6, 6]}
+    df = nw.from_native(constructor(data))
     if any(
         x in str(constructor)
         for x in ("pandas", "pandas[pyarrow]", "pandas[nullable]", "dask")
@@ -799,8 +801,6 @@ def test_join_duplicate_column_names(
         request.applymarker(pytest.mark.xfail)
     else:
         exception = nw.exceptions.DuplicateError
-    data = {"a": [1, 2, 3, 4, 5], "b": [6, 6, 6, 6, 6]}
-    df = nw.from_native(constructor(data))
     if isinstance(df, nw.LazyFrame):
         with pytest.raises(exception):
             df.join(df, on=["a"]).join(df, on=["a"]).collect()
